# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/noplay/code/gns3/gns3-gui/gns3/modules/dynamips/ui/ios_router_configuration_page.ui'
#
# Created by: PyQt5 UI code generator 5.4.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_iosRouterConfigPageWidget(object):

    def setupUi(self, iosRouterConfigPageWidget):
        iosRouterConfigPageWidget.setObjectName("iosRouterConfigPageWidget")
        iosRouterConfigPageWidget.resize(476, 510)
        self.vboxlayout = QtWidgets.QVBoxLayout(iosRouterConfigPageWidget)
        self.vboxlayout.setObjectName("vboxlayout")
        self.uiTabWidget = QtWidgets.QTabWidget(iosRouterConfigPageWidget)
        self.uiTabWidget.setObjectName("uiTabWidget")
        self.uiGeneralPageWidget = QtWidgets.QWidget()
        self.uiGeneralPageWidget.setObjectName("uiGeneralPageWidget")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.uiGeneralPageWidget)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.uiNameLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiNameLabel.setObjectName("uiNameLabel")
        self.gridLayout_2.addWidget(self.uiNameLabel, 0, 0, 1, 1)
        self.uiNameLineEdit = QtWidgets.QLineEdit(self.uiGeneralPageWidget)
        self.uiNameLineEdit.setObjectName("uiNameLineEdit")
        self.gridLayout_2.addWidget(self.uiNameLineEdit, 0, 1, 1, 1)
        self.uiPlatformLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiPlatformLabel.setObjectName("uiPlatformLabel")
        self.gridLayout_2.addWidget(self.uiPlatformLabel, 3, 0, 1, 1)
        self.uiPlatformTextLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiPlatformTextLabel.setText("")
        self.uiPlatformTextLabel.setObjectName("uiPlatformTextLabel")
        self.gridLayout_2.addWidget(self.uiPlatformTextLabel, 3, 1, 1, 1)
        self.uiChassisLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiChassisLabel.setObjectName("uiChassisLabel")
        self.gridLayout_2.addWidget(self.uiChassisLabel, 4, 0, 1, 1)
        self.uiChassisTextLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiChassisTextLabel.setText("")
        self.uiChassisTextLabel.setObjectName("uiChassisTextLabel")
        self.gridLayout_2.addWidget(self.uiChassisTextLabel, 4, 1, 1, 1)
        self.uiIOSImageLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiIOSImageLabel.setObjectName("uiIOSImageLabel")
        self.gridLayout_2.addWidget(self.uiIOSImageLabel, 8, 0, 1, 1)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.uiIOSImageLineEdit = QtWidgets.QLineEdit(self.uiGeneralPageWidget)
        self.uiIOSImageLineEdit.setObjectName("uiIOSImageLineEdit")
        self.horizontalLayout_5.addWidget(self.uiIOSImageLineEdit)
        self.uiIOSImageToolButton = QtWidgets.QToolButton(self.uiGeneralPageWidget)
        self.uiIOSImageToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiIOSImageToolButton.setObjectName("uiIOSImageToolButton")
        self.horizontalLayout_5.addWidget(self.uiIOSImageToolButton)
        self.gridLayout_2.addLayout(self.horizontalLayout_5, 8, 1, 1, 1)
        self.uiStartupConfigLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiStartupConfigLabel.setObjectName("uiStartupConfigLabel")
        self.gridLayout_2.addWidget(self.uiStartupConfigLabel, 9, 0, 1, 1)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.uiStartupConfigLineEdit = QtWidgets.QLineEdit(self.uiGeneralPageWidget)
        self.uiStartupConfigLineEdit.setObjectName("uiStartupConfigLineEdit")
        self.horizontalLayout_4.addWidget(self.uiStartupConfigLineEdit)
        self.uiStartupConfigToolButton = QtWidgets.QToolButton(self.uiGeneralPageWidget)
        self.uiStartupConfigToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiStartupConfigToolButton.setObjectName("uiStartupConfigToolButton")
        self.horizontalLayout_4.addWidget(self.uiStartupConfigToolButton)
        self.gridLayout_2.addLayout(self.horizontalLayout_4, 9, 1, 1, 1)
        self.uiPrivateConfigLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiPrivateConfigLabel.setObjectName("uiPrivateConfigLabel")
        self.gridLayout_2.addWidget(self.uiPrivateConfigLabel, 10, 0, 1, 1)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.uiPrivateConfigLineEdit = QtWidgets.QLineEdit(self.uiGeneralPageWidget)
        self.uiPrivateConfigLineEdit.setObjectName("uiPrivateConfigLineEdit")
        self.horizontalLayout_6.addWidget(self.uiPrivateConfigLineEdit)
        self.uiPrivateConfigToolButton = QtWidgets.QToolButton(self.uiGeneralPageWidget)
        self.uiPrivateConfigToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiPrivateConfigToolButton.setObjectName("uiPrivateConfigToolButton")
        self.horizontalLayout_6.addWidget(self.uiPrivateConfigToolButton)
        self.gridLayout_2.addLayout(self.horizontalLayout_6, 10, 1, 1, 1)
        self.uiConsolePortLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiConsolePortLabel.setObjectName("uiConsolePortLabel")
        self.gridLayout_2.addWidget(self.uiConsolePortLabel, 13, 0, 1, 1)
        self.uiConsolePortSpinBox = QtWidgets.QSpinBox(self.uiGeneralPageWidget)
        self.uiConsolePortSpinBox.setMaximum(65535)
        self.uiConsolePortSpinBox.setObjectName("uiConsolePortSpinBox")
        self.gridLayout_2.addWidget(self.uiConsolePortSpinBox, 13, 1, 1, 1)
        self.uiAuxPortLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiAuxPortLabel.setObjectName("uiAuxPortLabel")
        self.gridLayout_2.addWidget(self.uiAuxPortLabel, 14, 0, 1, 1)
        self.uiAuxPortSpinBox = QtWidgets.QSpinBox(self.uiGeneralPageWidget)
        self.uiAuxPortSpinBox.setMaximum(65535)
        self.uiAuxPortSpinBox.setObjectName("uiAuxPortSpinBox")
        self.gridLayout_2.addWidget(self.uiAuxPortSpinBox, 14, 1, 1, 1)
        self.uiMidplaneLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiMidplaneLabel.setObjectName("uiMidplaneLabel")
        self.gridLayout_2.addWidget(self.uiMidplaneLabel, 15, 0, 1, 1)
        self.uiMidplaneComboBox = QtWidgets.QComboBox(self.uiGeneralPageWidget)
        self.uiMidplaneComboBox.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiMidplaneComboBox.sizePolicy().hasHeightForWidth())
        self.uiMidplaneComboBox.setSizePolicy(sizePolicy)
        self.uiMidplaneComboBox.setObjectName("uiMidplaneComboBox")
        self.gridLayout_2.addWidget(self.uiMidplaneComboBox, 15, 1, 1, 1)
        self.uiNPELabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiNPELabel.setObjectName("uiNPELabel")
        self.gridLayout_2.addWidget(self.uiNPELabel, 16, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(263, 151, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem, 17, 1, 1, 1)
        self.uiNPEComboBox = QtWidgets.QComboBox(self.uiGeneralPageWidget)
        self.uiNPEComboBox.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiNPEComboBox.sizePolicy().hasHeightForWidth())
        self.uiNPEComboBox.setSizePolicy(sizePolicy)
        self.uiNPEComboBox.setObjectName("uiNPEComboBox")
        self.gridLayout_2.addWidget(self.uiNPEComboBox, 16, 1, 1, 1)
        self.uiSymbolLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiSymbolLabel.setObjectName("uiSymbolLabel")
        self.gridLayout_2.addWidget(self.uiSymbolLabel, 5, 0, 1, 1)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.uiSymbolLineEdit = QtWidgets.QLineEdit(self.uiGeneralPageWidget)
        self.uiSymbolLineEdit.setObjectName("uiSymbolLineEdit")
        self.horizontalLayout_7.addWidget(self.uiSymbolLineEdit)
        self.uiSymbolToolButton = QtWidgets.QToolButton(self.uiGeneralPageWidget)
        self.uiSymbolToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiSymbolToolButton.setObjectName("uiSymbolToolButton")
        self.horizontalLayout_7.addWidget(self.uiSymbolToolButton)
        self.gridLayout_2.addLayout(self.horizontalLayout_7, 5, 1, 1, 1)
        self.uiCategoryLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiCategoryLabel.setObjectName("uiCategoryLabel")
        self.gridLayout_2.addWidget(self.uiCategoryLabel, 6, 0, 1, 1)
        self.uiCategoryComboBox = QtWidgets.QComboBox(self.uiGeneralPageWidget)
        self.uiCategoryComboBox.setObjectName("uiCategoryComboBox")
        self.gridLayout_2.addWidget(self.uiCategoryComboBox, 6, 1, 1, 1)
        self.uiTabWidget.addTab(self.uiGeneralPageWidget, "")
        self.uiMemoriesPageWidget = QtWidgets.QWidget()
        self.uiMemoriesPageWidget.setObjectName("uiMemoriesPageWidget")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.uiMemoriesPageWidget)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.uiRamLabel = QtWidgets.QLabel(self.uiMemoriesPageWidget)
        self.uiRamLabel.setObjectName("uiRamLabel")
        self.gridLayout_5.addWidget(self.uiRamLabel, 0, 0, 1, 1)
        self.uiRamSpinBox = QtWidgets.QSpinBox(self.uiMemoriesPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRamSpinBox.sizePolicy().hasHeightForWidth())
        self.uiRamSpinBox.setSizePolicy(sizePolicy)
        self.uiRamSpinBox.setMinimum(32)
        self.uiRamSpinBox.setMaximum(65535)
        self.uiRamSpinBox.setSingleStep(32)
        self.uiRamSpinBox.setProperty("value", 128)
        self.uiRamSpinBox.setObjectName("uiRamSpinBox")
        self.gridLayout_5.addWidget(self.uiRamSpinBox, 0, 1, 1, 1)
        self.uiNvramLabel = QtWidgets.QLabel(self.uiMemoriesPageWidget)
        self.uiNvramLabel.setObjectName("uiNvramLabel")
        self.gridLayout_5.addWidget(self.uiNvramLabel, 1, 0, 1, 1)
        self.uiNvramSpinBox = QtWidgets.QSpinBox(self.uiMemoriesPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiNvramSpinBox.sizePolicy().hasHeightForWidth())
        self.uiNvramSpinBox.setSizePolicy(sizePolicy)
        self.uiNvramSpinBox.setMinimum(32)
        self.uiNvramSpinBox.setMaximum(65535)
        self.uiNvramSpinBox.setSingleStep(32)
        self.uiNvramSpinBox.setProperty("value", 128)
        self.uiNvramSpinBox.setObjectName("uiNvramSpinBox")
        self.gridLayout_5.addWidget(self.uiNvramSpinBox, 1, 1, 1, 1)
        self.uiIomemLabel = QtWidgets.QLabel(self.uiMemoriesPageWidget)
        self.uiIomemLabel.setObjectName("uiIomemLabel")
        self.gridLayout_5.addWidget(self.uiIomemLabel, 2, 0, 1, 1)
        self.uiIomemSpinBox = QtWidgets.QSpinBox(self.uiMemoriesPageWidget)
        self.uiIomemSpinBox.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiIomemSpinBox.sizePolicy().hasHeightForWidth())
        self.uiIomemSpinBox.setSizePolicy(sizePolicy)
        self.uiIomemSpinBox.setMaximum(100)
        self.uiIomemSpinBox.setSingleStep(5)
        self.uiIomemSpinBox.setProperty("value", 5)
        self.uiIomemSpinBox.setObjectName("uiIomemSpinBox")
        self.gridLayout_5.addWidget(self.uiIomemSpinBox, 2, 1, 1, 1)
        self.uiDisk0Label = QtWidgets.QLabel(self.uiMemoriesPageWidget)
        self.uiDisk0Label.setObjectName("uiDisk0Label")
        self.gridLayout_5.addWidget(self.uiDisk0Label, 3, 0, 1, 1)
        self.uiDisk0SpinBox = QtWidgets.QSpinBox(self.uiMemoriesPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiDisk0SpinBox.sizePolicy().hasHeightForWidth())
        self.uiDisk0SpinBox.setSizePolicy(sizePolicy)
        self.uiDisk0SpinBox.setMaximum(99999)
        self.uiDisk0SpinBox.setSingleStep(4)
        self.uiDisk0SpinBox.setObjectName("uiDisk0SpinBox")
        self.gridLayout_5.addWidget(self.uiDisk0SpinBox, 3, 1, 1, 1)
        self.uiDisk1Label = QtWidgets.QLabel(self.uiMemoriesPageWidget)
        self.uiDisk1Label.setObjectName("uiDisk1Label")
        self.gridLayout_5.addWidget(self.uiDisk1Label, 4, 0, 1, 1)
        self.uiDisk1SpinBox = QtWidgets.QSpinBox(self.uiMemoriesPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiDisk1SpinBox.sizePolicy().hasHeightForWidth())
        self.uiDisk1SpinBox.setSizePolicy(sizePolicy)
        self.uiDisk1SpinBox.setMaximum(99999)
        self.uiDisk1SpinBox.setSingleStep(4)
        self.uiDisk1SpinBox.setObjectName("uiDisk1SpinBox")
        self.gridLayout_5.addWidget(self.uiDisk1SpinBox, 4, 1, 1, 1)
        self.uiAutoDeleteCheckBox = QtWidgets.QCheckBox(self.uiMemoriesPageWidget)
        self.uiAutoDeleteCheckBox.setObjectName("uiAutoDeleteCheckBox")
        self.gridLayout_5.addWidget(self.uiAutoDeleteCheckBox, 5, 0, 1, 2)
        spacerItem1 = QtWidgets.QSpacerItem(20, 21, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_5.addItem(spacerItem1, 6, 0, 1, 1)
        self.uiTabWidget.addTab(self.uiMemoriesPageWidget, "")
        self.uiSlotsPageWidget = QtWidgets.QWidget()
        self.uiSlotsPageWidget.setObjectName("uiSlotsPageWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.uiSlotsPageWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiAdaptersGroupBox = QtWidgets.QGroupBox(self.uiSlotsPageWidget)
        self.uiAdaptersGroupBox.setObjectName("uiAdaptersGroupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.uiAdaptersGroupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.uiSlot1comboBox = QtWidgets.QComboBox(self.uiAdaptersGroupBox)
        self.uiSlot1comboBox.setObjectName("uiSlot1comboBox")
        self.gridLayout.addWidget(self.uiSlot1comboBox, 1, 1, 1, 1)
        self.uiSlot0Label = QtWidgets.QLabel(self.uiAdaptersGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot0Label.sizePolicy().hasHeightForWidth())
        self.uiSlot0Label.setSizePolicy(sizePolicy)
        self.uiSlot0Label.setObjectName("uiSlot0Label")
        self.gridLayout.addWidget(self.uiSlot0Label, 0, 0, 1, 1)
        self.uiSlot0comboBox = QtWidgets.QComboBox(self.uiAdaptersGroupBox)
        self.uiSlot0comboBox.setObjectName("uiSlot0comboBox")
        self.gridLayout.addWidget(self.uiSlot0comboBox, 0, 1, 1, 1)
        self.uiSlot1Label = QtWidgets.QLabel(self.uiAdaptersGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot1Label.sizePolicy().hasHeightForWidth())
        self.uiSlot1Label.setSizePolicy(sizePolicy)
        self.uiSlot1Label.setObjectName("uiSlot1Label")
        self.gridLayout.addWidget(self.uiSlot1Label, 1, 0, 1, 1)
        self.uiSlot2Label = QtWidgets.QLabel(self.uiAdaptersGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot2Label.sizePolicy().hasHeightForWidth())
        self.uiSlot2Label.setSizePolicy(sizePolicy)
        self.uiSlot2Label.setObjectName("uiSlot2Label")
        self.gridLayout.addWidget(self.uiSlot2Label, 2, 0, 1, 1)
        self.uiSlot2comboBox = QtWidgets.QComboBox(self.uiAdaptersGroupBox)
        self.uiSlot2comboBox.setObjectName("uiSlot2comboBox")
        self.gridLayout.addWidget(self.uiSlot2comboBox, 2, 1, 1, 1)
        self.uiSlot3Label = QtWidgets.QLabel(self.uiAdaptersGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot3Label.sizePolicy().hasHeightForWidth())
        self.uiSlot3Label.setSizePolicy(sizePolicy)
        self.uiSlot3Label.setObjectName("uiSlot3Label")
        self.gridLayout.addWidget(self.uiSlot3Label, 3, 0, 1, 1)
        self.uiSlot3comboBox = QtWidgets.QComboBox(self.uiAdaptersGroupBox)
        self.uiSlot3comboBox.setObjectName("uiSlot3comboBox")
        self.gridLayout.addWidget(self.uiSlot3comboBox, 3, 1, 1, 1)
        self.uiSlot4Label = QtWidgets.QLabel(self.uiAdaptersGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot4Label.sizePolicy().hasHeightForWidth())
        self.uiSlot4Label.setSizePolicy(sizePolicy)
        self.uiSlot4Label.setObjectName("uiSlot4Label")
        self.gridLayout.addWidget(self.uiSlot4Label, 4, 0, 1, 1)
        self.uiSlot4comboBox = QtWidgets.QComboBox(self.uiAdaptersGroupBox)
        self.uiSlot4comboBox.setObjectName("uiSlot4comboBox")
        self.gridLayout.addWidget(self.uiSlot4comboBox, 4, 1, 1, 1)
        self.uiSlot5Label = QtWidgets.QLabel(self.uiAdaptersGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot5Label.sizePolicy().hasHeightForWidth())
        self.uiSlot5Label.setSizePolicy(sizePolicy)
        self.uiSlot5Label.setObjectName("uiSlot5Label")
        self.gridLayout.addWidget(self.uiSlot5Label, 5, 0, 1, 1)
        self.uiSlot5comboBox = QtWidgets.QComboBox(self.uiAdaptersGroupBox)
        self.uiSlot5comboBox.setObjectName("uiSlot5comboBox")
        self.gridLayout.addWidget(self.uiSlot5comboBox, 5, 1, 1, 1)
        self.uiSlot6Label = QtWidgets.QLabel(self.uiAdaptersGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot6Label.sizePolicy().hasHeightForWidth())
        self.uiSlot6Label.setSizePolicy(sizePolicy)
        self.uiSlot6Label.setObjectName("uiSlot6Label")
        self.gridLayout.addWidget(self.uiSlot6Label, 6, 0, 1, 1)
        self.uiSlot6comboBox = QtWidgets.QComboBox(self.uiAdaptersGroupBox)
        self.uiSlot6comboBox.setObjectName("uiSlot6comboBox")
        self.gridLayout.addWidget(self.uiSlot6comboBox, 6, 1, 1, 1)
        self.verticalLayout.addWidget(self.uiAdaptersGroupBox)
        self.uiWicsGroupBox = QtWidgets.QGroupBox(self.uiSlotsPageWidget)
        self.uiWicsGroupBox.setObjectName("uiWicsGroupBox")
        self.gridlayout = QtWidgets.QGridLayout(self.uiWicsGroupBox)
        self.gridlayout.setObjectName("gridlayout")
        self.uiWic0Label = QtWidgets.QLabel(self.uiWicsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiWic0Label.sizePolicy().hasHeightForWidth())
        self.uiWic0Label.setSizePolicy(sizePolicy)
        self.uiWic0Label.setObjectName("uiWic0Label")
        self.gridlayout.addWidget(self.uiWic0Label, 0, 0, 1, 1)
        self.uiWic0comboBox = QtWidgets.QComboBox(self.uiWicsGroupBox)
        self.uiWic0comboBox.setObjectName("uiWic0comboBox")
        self.gridlayout.addWidget(self.uiWic0comboBox, 0, 1, 1, 1)
        self.uiWic1Label = QtWidgets.QLabel(self.uiWicsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiWic1Label.sizePolicy().hasHeightForWidth())
        self.uiWic1Label.setSizePolicy(sizePolicy)
        self.uiWic1Label.setObjectName("uiWic1Label")
        self.gridlayout.addWidget(self.uiWic1Label, 1, 0, 1, 1)
        self.uiWic1comboBox = QtWidgets.QComboBox(self.uiWicsGroupBox)
        self.uiWic1comboBox.setObjectName("uiWic1comboBox")
        self.gridlayout.addWidget(self.uiWic1comboBox, 1, 1, 1, 1)
        self.uiWic2Label = QtWidgets.QLabel(self.uiWicsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiWic2Label.sizePolicy().hasHeightForWidth())
        self.uiWic2Label.setSizePolicy(sizePolicy)
        self.uiWic2Label.setObjectName("uiWic2Label")
        self.gridlayout.addWidget(self.uiWic2Label, 2, 0, 1, 1)
        self.uiWic2comboBox = QtWidgets.QComboBox(self.uiWicsGroupBox)
        self.uiWic2comboBox.setObjectName("uiWic2comboBox")
        self.gridlayout.addWidget(self.uiWic2comboBox, 2, 1, 1, 1)
        self.verticalLayout.addWidget(self.uiWicsGroupBox)
        spacerItem2 = QtWidgets.QSpacerItem(325, 31, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem2)
        self.uiTabWidget.addTab(self.uiSlotsPageWidget, "")
        self.uiAdvancedPageWidget = QtWidgets.QWidget()
        self.uiAdvancedPageWidget.setObjectName("uiAdvancedPageWidget")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.uiAdvancedPageWidget)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.uiSystemGroupBox = QtWidgets.QGroupBox(self.uiAdvancedPageWidget)
        self.uiSystemGroupBox.setObjectName("uiSystemGroupBox")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.uiSystemGroupBox)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.uiSystemIdLineEdit = QtWidgets.QLineEdit(self.uiSystemGroupBox)
        self.uiSystemIdLineEdit.setObjectName("uiSystemIdLineEdit")
        self.gridLayout_6.addWidget(self.uiSystemIdLineEdit, 0, 1, 1, 1)
        self.label = QtWidgets.QLabel(self.uiSystemGroupBox)
        self.label.setObjectName("label")
        self.gridLayout_6.addWidget(self.label, 0, 0, 1, 1)
        self.uiBaseMacLabel = QtWidgets.QLabel(self.uiSystemGroupBox)
        self.uiBaseMacLabel.setObjectName("uiBaseMacLabel")
        self.gridLayout_6.addWidget(self.uiBaseMacLabel, 1, 0, 1, 1)
        self.uiBaseMACLineEdit = QtWidgets.QLineEdit(self.uiSystemGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiBaseMACLineEdit.sizePolicy().hasHeightForWidth())
        self.uiBaseMACLineEdit.setSizePolicy(sizePolicy)
        self.uiBaseMACLineEdit.setText("")
        self.uiBaseMACLineEdit.setObjectName("uiBaseMACLineEdit")
        self.gridLayout_6.addWidget(self.uiBaseMACLineEdit, 1, 1, 1, 1)
        self.verticalLayout_4.addWidget(self.uiSystemGroupBox)
        self.uiOptimizationsGroupBox = QtWidgets.QGroupBox(self.uiAdvancedPageWidget)
        self.uiOptimizationsGroupBox.setObjectName("uiOptimizationsGroupBox")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.uiOptimizationsGroupBox)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.uiExecAreaLabel = QtWidgets.QLabel(self.uiOptimizationsGroupBox)
        self.uiExecAreaLabel.setObjectName("uiExecAreaLabel")
        self.gridLayout_4.addWidget(self.uiExecAreaLabel, 3, 0, 1, 1)
        self.uiExecAreaSpinBox = QtWidgets.QSpinBox(self.uiOptimizationsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiExecAreaSpinBox.sizePolicy().hasHeightForWidth())
        self.uiExecAreaSpinBox.setSizePolicy(sizePolicy)
        self.uiExecAreaSpinBox.setMaximum(4096)
        self.uiExecAreaSpinBox.setSingleStep(4)
        self.uiExecAreaSpinBox.setProperty("value", 64)
        self.uiExecAreaSpinBox.setObjectName("uiExecAreaSpinBox")
        self.gridLayout_4.addWidget(self.uiExecAreaSpinBox, 3, 1, 1, 1)
        self.uiSparseMemoryCheckBox = QtWidgets.QCheckBox(self.uiOptimizationsGroupBox)
        self.uiSparseMemoryCheckBox.setChecked(False)
        self.uiSparseMemoryCheckBox.setObjectName("uiSparseMemoryCheckBox")
        self.gridLayout_4.addWidget(self.uiSparseMemoryCheckBox, 5, 0, 1, 2)
        self.uiIdlepcLabel = QtWidgets.QLabel(self.uiOptimizationsGroupBox)
        self.uiIdlepcLabel.setObjectName("uiIdlepcLabel")
        self.gridLayout_4.addWidget(self.uiIdlepcLabel, 0, 0, 1, 1)
        self.uiIdlepcLineEdit = QtWidgets.QLineEdit(self.uiOptimizationsGroupBox)
        self.uiIdlepcLineEdit.setObjectName("uiIdlepcLineEdit")
        self.gridLayout_4.addWidget(self.uiIdlepcLineEdit, 0, 1, 1, 1)
        self.uiIdlemaxLabel = QtWidgets.QLabel(self.uiOptimizationsGroupBox)
        self.uiIdlemaxLabel.setObjectName("uiIdlemaxLabel")
        self.gridLayout_4.addWidget(self.uiIdlemaxLabel, 1, 0, 1, 1)
        self.uiIdlemaxSpinBox = QtWidgets.QSpinBox(self.uiOptimizationsGroupBox)
        self.uiIdlemaxSpinBox.setMinimum(100)
        self.uiIdlemaxSpinBox.setMaximum(10000)
        self.uiIdlemaxSpinBox.setSingleStep(100)
        self.uiIdlemaxSpinBox.setProperty("value", 500)
        self.uiIdlemaxSpinBox.setObjectName("uiIdlemaxSpinBox")
        self.gridLayout_4.addWidget(self.uiIdlemaxSpinBox, 1, 1, 1, 1)
        self.uiIdlesleepLabel = QtWidgets.QLabel(self.uiOptimizationsGroupBox)
        self.uiIdlesleepLabel.setObjectName("uiIdlesleepLabel")
        self.gridLayout_4.addWidget(self.uiIdlesleepLabel, 2, 0, 1, 1)
        self.uiMmapCheckBox = QtWidgets.QCheckBox(self.uiOptimizationsGroupBox)
        self.uiMmapCheckBox.setChecked(True)
        self.uiMmapCheckBox.setObjectName("uiMmapCheckBox")
        self.gridLayout_4.addWidget(self.uiMmapCheckBox, 4, 0, 1, 2)
        self.uiIdlesleepSpinBox = QtWidgets.QSpinBox(self.uiOptimizationsGroupBox)
        self.uiIdlesleepSpinBox.setMinimum(1)
        self.uiIdlesleepSpinBox.setMaximum(500)
        self.uiIdlesleepSpinBox.setSingleStep(10)
        self.uiIdlesleepSpinBox.setProperty("value", 30)
        self.uiIdlesleepSpinBox.setObjectName("uiIdlesleepSpinBox")
        self.gridLayout_4.addWidget(self.uiIdlesleepSpinBox, 2, 1, 1, 1)
        self.verticalLayout_4.addWidget(self.uiOptimizationsGroupBox)
        spacerItem3 = QtWidgets.QSpacerItem(304, 251, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem3)
        self.uiTabWidget.addTab(self.uiAdvancedPageWidget, "")
        self.uiEnvironmentPageWidget = QtWidgets.QWidget()
        self.uiEnvironmentPageWidget.setObjectName("uiEnvironmentPageWidget")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.uiEnvironmentPageWidget)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.uiPowerSuppliesGroupBox = QtWidgets.QGroupBox(self.uiEnvironmentPageWidget)
        self.uiPowerSuppliesGroupBox.setObjectName("uiPowerSuppliesGroupBox")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.uiPowerSuppliesGroupBox)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.uiPowerSupply2Label = QtWidgets.QLabel(self.uiPowerSuppliesGroupBox)
        self.uiPowerSupply2Label.setObjectName("uiPowerSupply2Label")
        self.gridLayout_7.addWidget(self.uiPowerSupply2Label, 1, 0, 1, 1)
        self.uiPowerSupply1ComboBox = QtWidgets.QComboBox(self.uiPowerSuppliesGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiPowerSupply1ComboBox.sizePolicy().hasHeightForWidth())
        self.uiPowerSupply1ComboBox.setSizePolicy(sizePolicy)
        self.uiPowerSupply1ComboBox.setObjectName("uiPowerSupply1ComboBox")
        self.uiPowerSupply1ComboBox.addItem("")
        self.uiPowerSupply1ComboBox.addItem("")
        self.gridLayout_7.addWidget(self.uiPowerSupply1ComboBox, 0, 1, 1, 1)
        self.uiPowerSupply1Label = QtWidgets.QLabel(self.uiPowerSuppliesGroupBox)
        self.uiPowerSupply1Label.setObjectName("uiPowerSupply1Label")
        self.gridLayout_7.addWidget(self.uiPowerSupply1Label, 0, 0, 1, 1)
        self.uiPowerSupply2ComboBox = QtWidgets.QComboBox(self.uiPowerSuppliesGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiPowerSupply2ComboBox.sizePolicy().hasHeightForWidth())
        self.uiPowerSupply2ComboBox.setSizePolicy(sizePolicy)
        self.uiPowerSupply2ComboBox.setObjectName("uiPowerSupply2ComboBox")
        self.uiPowerSupply2ComboBox.addItem("")
        self.uiPowerSupply2ComboBox.addItem("")
        self.gridLayout_7.addWidget(self.uiPowerSupply2ComboBox, 1, 1, 1, 1)
        self.verticalLayout_3.addWidget(self.uiPowerSuppliesGroupBox)
        self.uiSensorsGroupBox = QtWidgets.QGroupBox(self.uiEnvironmentPageWidget)
        self.uiSensorsGroupBox.setObjectName("uiSensorsGroupBox")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.uiSensorsGroupBox)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.uiSensor1Label = QtWidgets.QLabel(self.uiSensorsGroupBox)
        self.uiSensor1Label.setObjectName("uiSensor1Label")
        self.gridLayout_3.addWidget(self.uiSensor1Label, 0, 0, 1, 1)
        self.uiSensor1SpinBox = QtWidgets.QSpinBox(self.uiSensorsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSensor1SpinBox.sizePolicy().hasHeightForWidth())
        self.uiSensor1SpinBox.setSizePolicy(sizePolicy)
        self.uiSensor1SpinBox.setMaximum(100)
        self.uiSensor1SpinBox.setObjectName("uiSensor1SpinBox")
        self.gridLayout_3.addWidget(self.uiSensor1SpinBox, 0, 1, 1, 1)
        self.uiSensor2Label = QtWidgets.QLabel(self.uiSensorsGroupBox)
        self.uiSensor2Label.setObjectName("uiSensor2Label")
        self.gridLayout_3.addWidget(self.uiSensor2Label, 1, 0, 1, 1)
        self.uiSensor2SpinBox = QtWidgets.QSpinBox(self.uiSensorsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSensor2SpinBox.sizePolicy().hasHeightForWidth())
        self.uiSensor2SpinBox.setSizePolicy(sizePolicy)
        self.uiSensor2SpinBox.setMaximum(100)
        self.uiSensor2SpinBox.setObjectName("uiSensor2SpinBox")
        self.gridLayout_3.addWidget(self.uiSensor2SpinBox, 1, 1, 1, 1)
        self.uiSensor3Label = QtWidgets.QLabel(self.uiSensorsGroupBox)
        self.uiSensor3Label.setObjectName("uiSensor3Label")
        self.gridLayout_3.addWidget(self.uiSensor3Label, 2, 0, 1, 1)
        self.uiSensor3SpinBox = QtWidgets.QSpinBox(self.uiSensorsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSensor3SpinBox.sizePolicy().hasHeightForWidth())
        self.uiSensor3SpinBox.setSizePolicy(sizePolicy)
        self.uiSensor3SpinBox.setMaximum(100)
        self.uiSensor3SpinBox.setObjectName("uiSensor3SpinBox")
        self.gridLayout_3.addWidget(self.uiSensor3SpinBox, 2, 1, 1, 1)
        self.uiSensor4Label = QtWidgets.QLabel(self.uiSensorsGroupBox)
        self.uiSensor4Label.setObjectName("uiSensor4Label")
        self.gridLayout_3.addWidget(self.uiSensor4Label, 3, 0, 1, 1)
        self.uiSensor4SpinBox = QtWidgets.QSpinBox(self.uiSensorsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSensor4SpinBox.sizePolicy().hasHeightForWidth())
        self.uiSensor4SpinBox.setSizePolicy(sizePolicy)
        self.uiSensor4SpinBox.setMaximum(100)
        self.uiSensor4SpinBox.setObjectName("uiSensor4SpinBox")
        self.gridLayout_3.addWidget(self.uiSensor4SpinBox, 3, 1, 1, 1)
        self.verticalLayout_3.addWidget(self.uiSensorsGroupBox)
        spacerItem4 = QtWidgets.QSpacerItem(20, 194, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem4)
        self.uiTabWidget.addTab(self.uiEnvironmentPageWidget, "")
        self.vboxlayout.addWidget(self.uiTabWidget)

        self.retranslateUi(iosRouterConfigPageWidget)
        self.uiTabWidget.setCurrentIndex(0)
        self.uiPowerSupply1ComboBox.setCurrentIndex(1)
        self.uiPowerSupply2ComboBox.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(iosRouterConfigPageWidget)
        iosRouterConfigPageWidget.setTabOrder(self.uiMidplaneComboBox, self.uiNPEComboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiNPEComboBox, self.uiSlot0comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiSlot0comboBox, self.uiSlot1comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiSlot1comboBox, self.uiSlot2comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiSlot2comboBox, self.uiSlot3comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiSlot3comboBox, self.uiSlot4comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiSlot4comboBox, self.uiSlot5comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiSlot5comboBox, self.uiSlot6comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiSlot6comboBox, self.uiWic0comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiWic0comboBox, self.uiWic1comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiWic1comboBox, self.uiWic2comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiWic2comboBox, self.uiBaseMACLineEdit)
        iosRouterConfigPageWidget.setTabOrder(self.uiBaseMACLineEdit, self.uiExecAreaSpinBox)

    def retranslateUi(self, iosRouterConfigPageWidget):
        _translate = QtCore.QCoreApplication.translate
        iosRouterConfigPageWidget.setWindowTitle(_translate("iosRouterConfigPageWidget", "Dynamips IOS Router configuration"))
        self.uiNameLabel.setText(_translate("iosRouterConfigPageWidget", "Name:"))
        self.uiPlatformLabel.setText(_translate("iosRouterConfigPageWidget", "Platform:"))
        self.uiChassisLabel.setText(_translate("iosRouterConfigPageWidget", "Chassis:"))
        self.uiIOSImageLabel.setText(_translate("iosRouterConfigPageWidget", "IOS image path:"))
        self.uiIOSImageToolButton.setText(_translate("iosRouterConfigPageWidget", "&Browse..."))
        self.uiStartupConfigLabel.setText(_translate("iosRouterConfigPageWidget", "Initial startup-config:"))
        self.uiStartupConfigToolButton.setText(_translate("iosRouterConfigPageWidget", "&Browse..."))
        self.uiPrivateConfigLabel.setText(_translate("iosRouterConfigPageWidget", "Initial private-config:"))
        self.uiPrivateConfigToolButton.setText(_translate("iosRouterConfigPageWidget", "&Browse..."))
        self.uiConsolePortLabel.setText(_translate("iosRouterConfigPageWidget", "Console port:"))
        self.uiAuxPortLabel.setText(_translate("iosRouterConfigPageWidget", "Aux port:"))
        self.uiMidplaneLabel.setText(_translate("iosRouterConfigPageWidget", "Midplane:"))
        self.uiNPELabel.setText(_translate("iosRouterConfigPageWidget", "NPE:"))
        self.uiSymbolLabel.setText(_translate("iosRouterConfigPageWidget", "Symbol:"))
        self.uiSymbolToolButton.setText(_translate("iosRouterConfigPageWidget", "&Browse..."))
        self.uiCategoryLabel.setText(_translate("iosRouterConfigPageWidget", "Category:"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiGeneralPageWidget), _translate("iosRouterConfigPageWidget", "General"))
        self.uiRamLabel.setText(_translate("iosRouterConfigPageWidget", "RAM size:"))
        self.uiRamSpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " MiB"))
        self.uiNvramLabel.setText(_translate("iosRouterConfigPageWidget", "NVRAM size:"))
        self.uiNvramSpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " KiB"))
        self.uiIomemLabel.setText(_translate("iosRouterConfigPageWidget", "I/O memory :"))
        self.uiIomemSpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " %"))
        self.uiDisk0Label.setText(_translate("iosRouterConfigPageWidget", "PCMCIA disk0:"))
        self.uiDisk0SpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " MiB"))
        self.uiDisk1Label.setText(_translate("iosRouterConfigPageWidget", "PCMCIA disk1:"))
        self.uiDisk1SpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " MiB"))
        self.uiAutoDeleteCheckBox.setText(_translate("iosRouterConfigPageWidget", "Automatically delete NVRAM and disk files"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiMemoriesPageWidget), _translate("iosRouterConfigPageWidget", "Memories and disks"))
        self.uiAdaptersGroupBox.setTitle(_translate("iosRouterConfigPageWidget", "Adapters"))
        self.uiSlot0Label.setText(_translate("iosRouterConfigPageWidget", "slot 0:"))
        self.uiSlot1Label.setText(_translate("iosRouterConfigPageWidget", "slot 1:"))
        self.uiSlot2Label.setText(_translate("iosRouterConfigPageWidget", "slot 2:"))
        self.uiSlot3Label.setText(_translate("iosRouterConfigPageWidget", "slot 3:"))
        self.uiSlot4Label.setText(_translate("iosRouterConfigPageWidget", "slot 4:"))
        self.uiSlot5Label.setText(_translate("iosRouterConfigPageWidget", "slot 5:"))
        self.uiSlot6Label.setText(_translate("iosRouterConfigPageWidget", "slot 6:"))
        self.uiWicsGroupBox.setTitle(_translate("iosRouterConfigPageWidget", "WICs"))
        self.uiWic0Label.setText(_translate("iosRouterConfigPageWidget", "wic 0:"))
        self.uiWic1Label.setText(_translate("iosRouterConfigPageWidget", "wic 1:"))
        self.uiWic2Label.setText(_translate("iosRouterConfigPageWidget", "wic 2:"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiSlotsPageWidget), _translate("iosRouterConfigPageWidget", "Slots"))
        self.uiSystemGroupBox.setTitle(_translate("iosRouterConfigPageWidget", "System"))
        self.label.setText(_translate("iosRouterConfigPageWidget", "System ID:"))
        self.uiBaseMacLabel.setText(_translate("iosRouterConfigPageWidget", "Base MAC:"))
        self.uiOptimizationsGroupBox.setTitle(_translate("iosRouterConfigPageWidget", "Optimisations"))
        self.uiExecAreaLabel.setText(_translate("iosRouterConfigPageWidget", "Exec area:"))
        self.uiExecAreaSpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " MiB"))
        self.uiSparseMemoryCheckBox.setText(_translate("iosRouterConfigPageWidget", "Enable sparse memory support"))
        self.uiIdlepcLabel.setText(_translate("iosRouterConfigPageWidget", "Idle-PC:"))
        self.uiIdlemaxLabel.setText(_translate("iosRouterConfigPageWidget", "Idlemax:"))
        self.uiIdlesleepLabel.setText(_translate("iosRouterConfigPageWidget", "Idlesleep:"))
        self.uiMmapCheckBox.setText(_translate("iosRouterConfigPageWidget", "Enable mmap support"))
        self.uiIdlesleepSpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " ms"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiAdvancedPageWidget), _translate("iosRouterConfigPageWidget", "Advanced"))
        self.uiPowerSuppliesGroupBox.setTitle(_translate("iosRouterConfigPageWidget", "Power supplies"))
        self.uiPowerSupply2Label.setText(_translate("iosRouterConfigPageWidget", "Power supply 2:"))
        self.uiPowerSupply1ComboBox.setItemText(0, _translate("iosRouterConfigPageWidget", "on"))
        self.uiPowerSupply1ComboBox.setItemText(1, _translate("iosRouterConfigPageWidget", "off"))
        self.uiPowerSupply1Label.setText(_translate("iosRouterConfigPageWidget", "Power supply 1:"))
        self.uiPowerSupply2ComboBox.setItemText(0, _translate("iosRouterConfigPageWidget", "on"))
        self.uiPowerSupply2ComboBox.setItemText(1, _translate("iosRouterConfigPageWidget", "off"))
        self.uiSensorsGroupBox.setTitle(_translate("iosRouterConfigPageWidget", "Temperature sensors"))
        self.uiSensor1Label.setText(_translate("iosRouterConfigPageWidget", "I/0 controller inlet:"))
        self.uiSensor1SpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " C"))
        self.uiSensor2Label.setText(_translate("iosRouterConfigPageWidget", "I/0 controller outlet:"))
        self.uiSensor2SpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " C"))
        self.uiSensor3Label.setText(_translate("iosRouterConfigPageWidget", "NPE inlet:"))
        self.uiSensor3SpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " C"))
        self.uiSensor4Label.setText(_translate("iosRouterConfigPageWidget", "NPE outlet:"))
        self.uiSensor4SpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " C"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiEnvironmentPageWidget), _translate("iosRouterConfigPageWidget", "Environment"))
